import stan
import matplotlib.pyplot as plt
import seaborn as sb
import arviz as az

binomial_model = """
    data {
        int N;
        int k;
    }
    parameters {
        real<lower = 0.0, upper = 1.0> p; 
    }
    model {
        k ~ binomial(N, p);
    }
"""

binomial_data = {'N':10, 'k':7}

sm = stan.build(binomial_model, data = binomial_data)
fit = sm.sample()

print('fit =\n', fit, '\n')

p_smpls = fit['p']
print('p_smpls =\n', p_smpls)

sb.kdeplot(p_smpls[0])
plt.savefig('FigKDE.png')
plt.show()
plt.close()

df = fit.to_frame()
sb.kdeplot(df['p'])
plt.savefig('FigPD.png')
plt.show()
plt.close()

inference_data = az.from_pystan(posterior = fit, posterior_model = sm)
az.plot_trace(inference_data)
plt.savefig('FigTrace.png')
plt.show()
plt.close()

