import stan
import scipy.stats as ss
import matplotlib.pyplot as plt
import seaborn as sb
import arviz as az

X = ss.multivariate_normal.rvs([0,0],[[1, 0.8],[0.8,1]], size = 100)
print(X)
plt.scatter(X.T[0], X.T[1])
plt.savefig('FigXY.png')
plt.show()
plt.close()

binormal_model = """
    data {
        int N;
        vector[2] X[N];
    }
    parameters {
        vector[2] mu;
        vector[2] sgm;
        real<lower = -1, upper = 1> rho;
    }
    transformed parameters {
        cov_matrix[2] cov;
        cov[1][1] = sgm[1]*sgm[1];
        cov[1][2] = rho*sgm[1]*sgm[2];
        cov[2][1] = rho*sgm[1]*sgm[2];
        cov[2][2] = sgm[2]*sgm[2];
    }
    model {
        mu[1] ~ normal(0, 10);
        mu[2] ~ normal(0, 10);
        sgm ~ exponential(0.1);
        rho ~ uniform(-1,1);
        X ~ multi_normal(mu, cov);
    }
"""

binormal_data = {'N':len(X), 'X':X}

sm = stan.build(binormal_model, data = binormal_data)
fit = sm.sample()

print('fit =\n', fit, '\n')

inference_data = az.from_pystan(posterior = fit, posterior_model = sm)
smry = az.summary(inference_data)
print('Summary =\n',smry)

az.plot_trace(inference_data)
plt.savefig('FigTrace.png')
print('FigTrace.png was saved.')
plt.show()
plt.close()

mu_smpls = fit['mu']
print('mu_smpls =\n', mu_smpls)

df = fit.to_frame()
print('df =\n', df, '\n\n')
for v in zip(df['mu.1'][:5], df['mu.2'][:5], df['cov.2.2'][:5]):
    print(v)
    
sb.kdeplot(df['rho'])
plt.savefig('FigRho.png')
plt.show()
plt.close()


