import stan
import scipy.stats as ss
import matplotlib.pyplot as plt
import seaborn as sb
import arviz as az

Xs = ss.uniform.rvs(loc = 10, scale = 20, size = 100)
Es = ss.norm.rvs(loc = 0, scale = 1, size = 100)
Ys = 10 + 0.5 * Xs + Es

plt.scatter(Xs, Ys)
plt.savefig('FigXY.png')
plt.show()
plt.close()

simple_reg_model = """
    data {
        int N;
        vector[N] X;
        vector[N] Y;
    }
    parameters {
        real a;
        real b;
        real<lower = 0> sgm;
    }
    model {
        a ~ normal(0, 10);
        b ~ normal(0, 10);
        sgm ~ exponential(0.1);
        Y ~ normal(a + b*X, sgm);
    }
"""

simple_reg_data = {'N':len(Xs), 'X':Xs, 'Y':Ys}

sm = stan.build(simple_reg_model, data = simple_reg_data)
fit = sm.sample()

print('fit =\n', fit, '\n')

infer_data = az.from_pystan(posterior=fit, posterior_model=sm)
smry = az.summary(infer_data)
print(smry)

az.plot_trace(infer_data)
plt.savefig('FigTrace.png')
print('FigTrace.png was saved.')
plt.show()
plt.close()

a_smpls = fit['a']
b_smpls = fit['b']
plt.scatter(a_smpls, b_smpls)
plt.xlabel('a')
plt.ylabel('b')
plt.savefig('Fig_ab.png')
plt.show()
plt.close()



